==> flambeau-0.4.0.tgz: package/lib/Flambeau.js <==
"use strict";

var _interopRequire = function (obj) { return obj && obj.__esModule ? obj["default"] : obj; };

var _prototypeProperties = function (child, staticProps, instanceProps) { if (staticProps) Object.defineProperties(child, staticProps); if (instanceProps) Object.defineProperties(child.prototype, instanceProps); };

var _classCallCheck = function (instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } };

var callAction = _interopRequire(require("./callAction"));

var _types = require("./types");

var ACTION_TYPE = _types.ACTION_TYPE;
var INTROSPECTION_TYPE = _types.INTROSPECTION_TYPE;

var SimpleGraphController = _interopRequire(require("./SimpleGraphController"));

var notFoundValue = {};

var Flambeau = (function () {
  function Flambeau() {
    var graph = arguments[0] === undefined ? new SimpleGraphController() : arguments[0];

    _classCallCheck(this, Flambeau);

    this.resources = {};
    this.graph = graph;
    this.actionSetIDsToActionFunctions = {};
    this.listeners = [];
  }

  _prototypeProperties(Flambeau, null, {
    attachReducer: {
      value: function attachReducer(id, reducer, props) {
        this.resources[id] = {
          reducer: reducer,
          props: props
        };

        this.graph = this.graph.setUp(id, {
          state: reducer.getInitialState(props)
        });
      },
      writable: true,
      configurable: true
    },
    attachReducers: {
      value: function attachReducers(idToReducers) {
        var _this = this;

        var idToProps = arguments[1] === undefined ? {} : arguments[1];

        Object.keys(idToReducers).forEach(function (id) {
          _this.attachReducer(id, idToReducers[id], idToProps[id]);
        });
      },
      writable: true,
      configurable: true
    },
    dispatch: {
      value: function dispatch(_ref) {
        var _this = this;

        var actionSetID = _ref.actionSetID;
        var actionID = _ref.actionID;
        var payload = _ref.payload;

        var idsChanged = {};

        Object.keys(this.resources).forEach(function (resourceID) {
          var resource = _this.resources[resourceID];

          var result = callAction({
            responder: resource.reducer,
            type: ACTION_TYPE,
            initialValue: _this.graph.get(resourceID),
            props: resource.props,
            payload: payload,
            notFoundValue: notFoundValue,
            actionSetID: actionSetID,
            actionID: actionID
          });

          if (result !== notFoundValue) {
            _this.graph = _this.graph.set(resourceID, result);
            idsChanged[resourceID] = true;
          }
        });

        this.listeners.forEach(function (callback) {
          callback(idsChanged);
        });
      },
      writable: true,
      configurable: true
    },
    registerActionSets: {
      value: function registerActionSets(actionSets) {
        var _this = this;

        Object.keys(actionSets).forEach(function (actionSetID) {
          _this.actionSetIDsToActionFunctions[actionSetID] = actionSets[actionSetID];
        });

        delete this.allConnectedActionSets;
      },
      writable: true,
      configurable: true
    },
    getConnectedActionSet: {
      value: function getConnectedActionSet(actionSetID) {
        var _this = this;

        var sourceActionFunctions = this.actionSetIDsToActionFunctions[actionSetID];
        var connectedActionFunctions = {};

        Object.keys(sourceActionFunctions).forEach(function (actionID) {
          if (actionID === "introspection") {
            return;
          }

          var sourceActionFunction = sourceActionFunctions[actionID];

          connectedActionFunctions[actionID] = function (payload) {
            // Synchronous, immediately dispatched
            if (sourceActionFunction.length <= 1) {
              var result = sourceActionFunction(payload);
              // Allow empty action declarations
              if (typeof result !== "undefined") {
                payload = result;
              }
              _this.dispatch({ actionSetID: actionSetID, actionID: actionID, payload: payload });
            }
            // Asychronous, delegates the dispatching
            else {
              var allActionSets = _this.getAllConnectedActionSets();
              var currentActionSet = allActionSets[actionSetID];

              var getConsensus = function (_ref) {
                var viewpointID = _ref.viewpointID;
                var payload = _ref.payload;
                var combine = _ref.combine;
                var _ref$booleanOr = _ref.booleanOr;
                var booleanOr = _ref$booleanOr === undefined ? false : _ref$booleanOr;
                var _ref$booleanAnd = _ref.booleanAnd;
                var booleanAnd = _ref$booleanAnd === undefined ? false : _ref$booleanAnd;

                if (booleanOr) {
                  combine = function (combined, current) {
                    return combined || current;
                  };
                } else if (booleanAnd) {
                  combine = function (combined, current) {
                    return combined && current;
                  };
                }

                return Object.keys(_this.resources).reduce(function (combinedValue, resourceID, i) {
                  var resource = _this.resources[resourceID];
                  var currentValue = callAction({
                    responder: resource.reducer,
                    type: INTROSPECTION_TYPE,
                    initialValue: _this.graph.get(resourceID),
                    defaultValue: notFoundValue,
                    props: resource.props,
                    actionID: viewpointID,
                    actionSetID: actionSetID,
                    payload: payload,
                    notFoundValue: notFoundValue
                  });

                  if (i === 0) {
                    combinedValue = currentValue;
                  } else {
                    combinedValue = combine(combinedValue, currentValue);
                  }

                  return combinedValue;
                }, undefined);
              };

              sourceActionFunction(payload, { currentActionSet: currentActionSet, allActionSets: allActionSets, getConsensus: getConsensus });
            }
          };
        });

        return connectedActionFunctions;
      },
      writable: true,
      configurable: true
    },
    getConnectedActionSets: {
      value: function getConnectedActionSets(actionSetIDs) {
        var _this = this;

        return actionSetIDs.reduce(function (connectedActionSets, actionSetID) {
          connectedActionSets[actionSetID] = _this.getConnectedActionSet(actionSetID);
          return connectedActionSets;
        }, {});
      },
      writable: true,
      configurable: true
    },
    getAllConnectedActionSets: {
      value: function getAllConnectedActionSets() {
        if (this.allConnectedActionSets) {
          return this.allConnectedActionSets;
        } else {
          this.allConnectedActionSets = this.getConnectedActionSets(Object.keys(this.actionSetIDsToActionFunctions));
          return this.allConnectedActionSets;
        }
      },
      writable: true,
      configurable: true
    },
    registerAndConnectActionSets: {
      value: function registerAndConnectActionSets(actionSets) {
        this.registerActionSets(actionSets);
        return this.getConnectedActionSets(Object.keys(actionSets));
      },
      writable: true,
      configurable: true
    },
    get: {
      value: function get(id) {
        var _this = this;

        if (typeof id === "undefined") {
          return this.graph.get();
        } else if (typeof id === "string") {
          return this.graph.get(id);
        } else {
          var _ret = (function () {
            var idToState = {};
            Object.keys(id).forEach(function (id) {
              idToState[id] = _this.graph.get(id);
            });
            return {
              v: idToState
            };
          })();

          if (typeof _ret === "object") {
            return _ret.v;
          }
        }
      },
      writable: true,
      configurable: true
    },
    subscribe: {
      value: function subscribe(callback) {
        var _this = this;

        this.listeners.push(callback);

        return function () {
          _this.listeners.splice(_this.listeners.indexOf(callback), 1);
        };
      },
      writable: true,
      configurable: true
    }
  });

  return Flambeau;
})();

module.exports = Flambeau;

==> flambeau-0.6.0.tgz: package/lib/Flambeau.js <==
"use strict";

var _interopRequire = function (obj) { return obj && obj.__esModule ? obj["default"] : obj; };

var _prototypeProperties = function (child, staticProps, instanceProps) { if (staticProps) Object.defineProperties(child, staticProps); if (instanceProps) Object.defineProperties(child.prototype, instanceProps); };

var _classCallCheck = function (instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } };

var callAction = _interopRequire(require("./callAction"));

var _types = require("./types");

var ACTION_TYPE = _types.ACTION_TYPE;
var INTROSPECTION_TYPE = _types.INTROSPECTION_TYPE;

var SimpleGraphController = _interopRequire(require("./SimpleGraphController"));

var notFoundValue = {};

var Flambeau = (function () {
  /**
   * Creates a new Flambeau store
   * @param  {graph controller} graph The object graph that reducers are attached to.
   */

  function Flambeau() {
    var graph = arguments[0] === undefined ? new SimpleGraphController() : arguments[0];

    _classCallCheck(this, Flambeau);

    this.resources = {};
    this.graph = graph;
    this.actionSetIDsToActionFunctions = {};
    this.listeners = [];
  }

  _prototypeProperties(Flambeau, null, {
    attachReducer: {

      /**
       * Attach a set of reducer functions.
       *
       * @param  {String} id      The unique identifier for this particular reducer
       * @param  {Object} reducer The set of reducer functions, most likely made using `import * as ...`
       * @param  {Object} props   The unique attributes to customize the reducer
       */

      value: function attachReducer(id, reducer, props) {
        this.resources[id] = {
          reducer: reducer,
          props: props
        };

        this.graph = this.graph.setUp(id, {
          state: reducer.getInitialState(props)
        });
      },
      writable: true,
      configurable: true
    },
    attachReducers: {

      /**
       * Attach multiple reducers, with ids as keys, and reducer function sets as values.
       *
       * @param  {Object} idToReducers An object mapping identifiers to reducer function sets
       * @param  {Object} idToProps    (Optional) the unique attributes for each particular reducer
       */

      value: function attachReducers(idToReducers) {
        var _this = this;

        var idToProps = arguments[1] === undefined ? {} : arguments[1];

        Object.keys(idToReducers).forEach(function (id) {
          _this.attachReducer(id, idToReducers[id], idToProps[id]);
        });
      },
      writable: true,
      configurable: true
    },
    dispatch: {
      value: function dispatch(_ref) {
        var _this = this;

        var actionSetID = _ref.actionSetID;
        var actionID = _ref.actionID;
        var payload = _ref.payload;

        var idsChanged = {};

        Object.keys(this.resources).forEach(function (resourceID) {
          var resource = _this.resources[resourceID];

          var result = callAction({
            responder: resource.reducer,
            type: ACTION_TYPE,
            initialValue: _this.graph.get(resourceID),
            props: resource.props,
            payload: payload,
            notFoundValue: notFoundValue,
            actionSetID: actionSetID,
            actionID: actionID
          });

          if (result !== notFoundValue) {
            _this.graph = _this.graph.set(resourceID, result);
            idsChanged[resourceID] = true;
          }
        });

        this.listeners.forEach(function (callback) {
          callback(idsChanged);
        });
      },
      writable: true,
      configurable: true
    },
    registerActionSets: {

      /**
       * Add actions that will be connected to reducers.
       *
       * @param  {Object} actionSets An object mapping action set identifiers to action functions
       */

      value: function registerActionSets(actionSets) {
        var _this = this;

        Object.keys(actionSets).forEach(function (actionSetID) {
          _this.actionSetIDsToActionFunctions[actionSetID] = actionSets[actionSetID];
        });

        delete this.allConnectedActionSets;
      },
      writable: true,
      configurable: true
    },
    getConnectedActionSet: {

      /**
       * Get a set of actions, connected to this store ready to dispatch to the reducers.
       *
       * @param  {String} actionSetID The unique identifier for the action set
       * @return {Object}             The connected action functions
       */

      value: function getConnectedActionSet(actionSetID) {
        var _this = this;

        var sourceActionFunctions = this.actionSetIDsToActionFunctions[actionSetID];
        var connectedActionFunctions = {};

        Object.keys(sourceActionFunctions).forEach(function (actionID) {
          if (actionID === "introspection") {
            return;
          }

          var sourceActionFunction = sourceActionFunctions[actionID];

          connectedActionFunctions[actionID] = function (payload) {
            // Synchronous, immediately dispatched
            if (sourceActionFunction.length <= 1) {
              var result = sourceActionFunction(payload);
              // Allow empty action declarations
              if (typeof result !== "undefined") {
                payload = result;
              }
              _this.dispatch({ actionSetID: actionSetID, actionID: actionID, payload: payload });
            }
            // Asychronous, delegates the dispatching
            else {
              var allActionSets = _this.getAllConnectedActionSets();
              var currentActionSet = allActionSets[actionSetID];

              var getConsensus = function (_ref) {
                var introspectionID = _ref.introspectionID;
                var payload = _ref.payload;
                var combine = _ref.combine;
                var _ref$booleanOr = _ref.booleanOr;
                var booleanOr = _ref$booleanOr === undefined ? false : _ref$booleanOr;
                var _ref$booleanAnd = _ref.booleanAnd;
                var booleanAnd = _ref$booleanAnd === undefined ? false : _ref$booleanAnd;

                if (booleanOr) {
                  combine = function (combined, current) {
                    return combined || current;
                  };
                } else if (booleanAnd) {
                  combine = function (combined, current) {
                    return combined && current;
                  };
                }

                var combinedValue = Object.keys(_this.resources).reduce(function (combinedValue, resourceID, i) {
                  var resource = _this.resources[resourceID];
                  var currentValue = callAction({
                    responder: resource.reducer,
                    type: INTROSPECTION_TYPE,
                    initialValue: _this.graph.get(resourceID),
                    props: resource.props,
                    actionID: introspectionID,
                    actionSetID: actionSetID,
                    payload: payload,
                    notFoundValue: notFoundValue
                  });

                  if (currentValue !== notFoundValue) {
                    if (combinedValue === notFoundValue) {
                      return currentValue;
                    } else {
                      return combine(combinedValue, currentValue);
                    }
                  }

                  return combinedValue;
                }, notFoundValue);

                if (combinedValue !== notFoundValue) {
                  return combinedValue;
                }
              };

              sourceActionFunction(payload, { currentActionSet: currentActionSet, allActionSets: allActionSets, getConsensus: getConsensus });
            }
          };
        });

        return connectedActionFunctions;
      },
      writable: true,
      configurable: true
    },
    getConnectedActionSets: {

      /**
       * Get multiple actions sets, connected to this store ready to dispatch to the reducers.
       *
       * @param  {Array} actionSetIDs The unique identifiers of the action sets.
       * @return {Object}             The action set identifiers to connected action function sets.
       */

      value: function getConnectedActionSets(actionSetIDs) {
        var _this = this;

        return actionSetIDs.reduce(function (connectedActionSets, actionSetID) {
          connectedActionSets[actionSetID] = _this.getConnectedActionSet(actionSetID);
          return connectedActionSets;
        }, {});
      },
      writable: true,
      configurable: true
    },
    getAllConnectedActionSets: {

      /**
       * Get all known action sets, connected to this store.
       *
       * @return {Object} The action set identifiers to action function sets.
       */

      value: function getAllConnectedActionSets() {
        if (this.allConnectedActionSets) {
          return this.allConnectedActionSets;
        } else {
          this.allConnectedActionSets = this.getConnectedActionSets(Object.keys(this.actionSetIDsToActionFunctions));
          return this.allConnectedActionSets;
        }
      },
      writable: true,
      configurable: true
    },
    registerAndConnectActionSets: {

      /**
       * Convenience method to register and connect action sets in one go.
       *
       * @param  {Object} actionSets An object mapping action set identifiers to action functions
       * @return {Object}            The action set identifiers to connected action function sets.
       */

      value: function registerAndConnectActionSets(actionSets) {
        this.registerActionSets(actionSets);
        return this.getConnectedActionSets(Object.keys(actionSets));
      },
      writable: true,
      configurable: true
    },
    get: {

      /**
       * Get the state for a particular reducer.
       *
       * @param  {String} id The unique identifier for a particular reducer
       * @return {Any}    The state value of the reducer at the current time
       */

      value: function get(id) {
        var _this = this;

        if (typeof id === "undefined") {
          return this.graph.get();
        } else if (typeof id === "string") {
          return this.graph.get(id);
        } else {
          var _ret = (function () {
            var idToState = {};
            Object.keys(id).forEach(function (id) {
              idToState[id] = _this.graph.get(id);
            });
            return {
              v: idToState
            };
          })();

          if (typeof _ret === "object") {
            return _ret.v;
          }
        }
      },
      writable: true,
      configurable: true
    },
    subscribe: {

      /**
       * Subscribe to reducer updates.
       *
       * @param  {Function} callback A function accepting an array of changed reducer identifiers
       * @return {Function}          A function used to unsubscribe when finished
       */

      value: function subscribe(callback) {
        var _this = this;

        this.listeners.push(callback);

        return function () {
          _this.listeners.splice(_this.listeners.indexOf(callback), 1);
        };
      },
      writable: true,
      configurable: true
    }
  });

  return Flambeau;
})();

module.exports = Flambeau;
